<?php
namespace KnitPay\Extensions\PaidMembershipsPro;

use Pronamic\WordPress\Money\Money;
use Pronamic\WordPress\Money\TaxedMoney;
use Pronamic\WordPress\Money\Parser as MoneyParser;
use Pronamic\WordPress\Pay\Payments\PaymentLines;
use Pronamic\WordPress\Pay\Payments\PaymentData as Pay_PaymentData;
use Pronamic\WordPress\Pay\Payments\Item;
use Pronamic\WordPress\Pay\Payments\Items;
use Pronamic\WordPress\Pay\Payments\PaymentDataInterface as Pronamic_Pay_PaymentDataInterface;

/**
 * Title: Paid Memberships Pro extension
 * Description:
 * Copyright: 2020 Knit Pay
 * Company: Knit Pay
 *
 * @author knitpay
 * @since 2.0.0
 */
class PaymentData extends Pay_PaymentData {


	/**
	 * The order ID
	 *
	 * @var \MemberOrder
	 */
	private $morder;

	/**
	 * Constructs and initializes an LearnPress payment data object.
	 *
	 * @param \MemberOrder $morder
	 *            Instance MemberOrder for the order being processed
	 */
	public function __construct( $morder ) {
		$this->morder   = $morder;
		$this->order_id = $this->morder->id;

		if ( isset( $this->morder->billing ) ) {
			$this->billing = $this->morder->billing;
		}

		parent::__construct();
	}

	/**
	 * Get source indicator
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_source()
	 * @return string
	 */
	public function get_source() {
		return 'paid-memberships-pro';
	}

	/**
	 * Get source ID.
	 *
	 * @return string
	 */
	public function get_source_id() {
		return $this->order_id;
	}

	/**
	 * Get title.
	 *
	 * @return string
	 */
	public function get_title() {
		/* translators: %s: order id */
		return sprintf( __( 'Paid Memberships Pro %s', 'pronamic_ideal' ), $this->get_order_id() );
	}

	/**
	 * Get description
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_description()
	 * @return string
	 */
	public function get_description() {
		$description = pmpro_getOption( 'knit_pay_payment_description' );
		if ( '' === $description ) {
			$description = $this->get_title();
		}

		// Replacements.
		$replacements = array(
			'{order_id}'        => $this->get_order_id(),
			'{code}'            => $this->morder->code,
			'{invoice_id}'      => $this->morder->code,
			'{membership_name}' => $this->morder->membership_name,
		);

		return strtr( $description, $replacements );
	}

	/**
	 * Get order ID
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_order_id()
	 * @return string
	 */
	public function get_order_id() {
		return $this->order_id;
	}

	/**
	 * Get items
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_items()
	 * @return Items
	 */
	public function get_items() {   }

	public function get_amount() {
		$initial_payment     = $this->morder->InitialPayment;
		$initial_payment_tax = $this->morder->getTaxForPrice( $initial_payment );
		$initial_payment     = pmpro_round_price( (float) $initial_payment + (float) $initial_payment_tax );

		$amount = new TaxedMoney( $initial_payment, $this->get_currency_alphabetic_code(), null, $initial_payment_tax );

		return $amount;
	}

	/**
	 * Get currency
	 *
	 * @see Pronamic_Pay_PaymentDataInterface::get_currency_alphabetic_code()
	 * @return string
	 */
	public function get_currency_alphabetic_code() {
		global $pmpro_currency;
		return $pmpro_currency;
	}

	/**
	 * Get email.
	 *
	 * @return string|null
	 */
	public function get_email() {
		if ( isset( $this->morder->Email ) ) {
			return $this->morder->Email;
		}
		if ( isset( $_POST['bemail'] ) ) {
			return $_POST['bemail'];
		}
		return null;
	}

	/**
	 * Get first name.
	 *
	 * @return string
	 */
	public function get_first_name() {
		if ( isset( $this->morder->FirstName ) ) {
			return $this->morder->FirstName;
		}
	}

	/**
	 * Get last name.
	 *
	 * @return string
	 */
	public function get_last_name() {
		if ( isset( $this->morder->LastName ) ) {
			return $this->morder->LastName;
		}
	}

	/**
	 * Get customer name.
	 *
	 * @return string
	 */
	public function get_customer_name() {
		return $this->billing->name;
	}

	/**
	 * Get address.
	 *
	 * @return null|string
	 */
	public function get_address() {
		return $this->billing->street;
	}

	/**
	 * Get city.
	 *
	 * @return null|string
	 */
	public function get_city() {
		return $this->billing->city;
	}

	/**
	 * Get ZIP.
	 *
	 * @return null|string
	 */
	public function get_zip() {
		return $this->billing->zip;
	}

	/**
	 * Get telephone number.
	 *
	 * @return null|string
	 */
	public function get_telephone_number() {
		return $this->billing->phone;
	}

	/**
	 * Get normal return URL.
	 *
	 * @link https://github.com/woothemes/woocommerce/blob/v2.1.3/includes/abstracts/abstract-wc-payment-gateway.php#L52
	 * @return string
	 */
	public function get_normal_return_url() {
		ddd();
		ppp( esc_url( $this->gateway->get_return_url( $this->order ) ) );
		ddd();
		return esc_url( $this->gateway->get_return_url( $this->order ) );

		// TODO:
		return charitable_get_permalink(
			'donation_receipt_page',
			array(
				'donation_id' => $this->donation_id,
			)
		);
	}

	/**
	 * Get cancel URL.
	 *
	 * @return string
	 */
	public function get_cancel_url() {
		ddd();
		return esc_url( learn_press_is_enable_cart() ? learn_press_get_page_link( 'cart' ) : get_home_url() /* SITE_URL */ );
		$cancel_url = charitable_get_permalink(
			'donation_cancel_page',
			array(
				'donation_id' => $this->donation_id,
			)
		);

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}

	/**
	 * Get success URL.
	 *
	 * @return string
	 */
	public function get_success_url() {
		ddd();
		return esc_url( $this->gateway->get_return_url( $this->order ) );
	}

	/**
	 * Get error URL.
	 *
	 * @return string
	 */
	public function get_error_url() {
		ddd();
		return esc_url( learn_press_is_enable_cart() ? learn_press_get_page_link( 'cart' ) : get_home_url() /* SITE_URL */ );
		$cancel_url = charitable_get_permalink(
			'donation_cancel_page',
			array(
				'donation_id' => $this->donation_id,
			)
		);

		if ( ! $cancel_url ) {
			$cancel_url = esc_url_raw(
				add_query_arg(
					array(
						'donation_id' => $this->donation_id,
						'cancel'      => true,
					),
					wp_get_referer()
				)
			);
		}

		return $cancel_url;
	}
}
